% bertragungsfunktion durch Rauschen 

%Shaker-Regelung mit Sinus are free software.
%you can redistribute it and/or under terms of the GNU General Public
%License as published by the Free Software Foundation.
%Shaker-Regelung mit Sinus are distributed in the hope that will useful,
%but without any warranty, without even the implied warranty of
%merchantability or fitness for a particular purpose. See the GNU for more
%details:
%Florian Fltzinger
%Hochschule fr angewandte Wissenschaften Mnchen
%University of applied sciences Munich

% Mit diesem Programm lsst sich die optimale Startamplitude fr die
% Shakerregelung bestimmen. Es wird ein Rauschsignal auf die Regelstrecke
% (bzw. Shaker) ausgegeben. Durch die gemessene Antwort der Sensoren kann
% die bertragungsfunktion ermittelt werden. Der Benutzer kann nun seine
% Regelparameter eingeben und bekommt Rckmeldung, ob die Regelung so
% berhaupt durchfhrbar ist und wenn ja, die optimale Startamplitude bei
% den Einstellungen am Leistungsverstrker und mit den Eingabeparametern.

s = daq.createSession('ni');

addAnalogInputChannel(s,'PXI1Slot2','ai2','Voltage');   %analog Eingang Beschleunigung
addAnalogInputChannel(s,'PXI1Slot2','ai3','Voltage');   %analog Eingang Kraft

addAnalogOutputChannel(s,'PXI1Slot2','ao0','Voltage');  %analog Ausgang

s.Rate = 100000;


output_data = 2*1*(rand(100000*20,1)*0.5);             %20 Sekunden Rauschsignal mit einer +- 1 Amplitude und einem Vektor 100000*20


queueOutputData(s,repmat([output_data],1,1));




duration = s.DurationInSeconds;

[beschl_kraft,time] = s.startForeground();             %Starten der Ausgabe

%Kalibrierwerte
B=(beschl_kraft(:,1))./(10.48*10*0.001);
F=(beschl_kraft(:,2))./(11.241*100*0.001);



figure(1);                                             %Plotten des Ausganssignals
plot(output_data);
title('Output Data Queued');
grid on;
grid minor;
hold on;

figure(2);                                             %Plotten der Messsignale Beschleunigung und Kraft
plot(time,beschl_kraft);
ylabel('Voltage');
xlabel('Time');
title('gemessene Beschleunigung und Kraft');
grid on;
grid minor;

figure(3);                                             %Plotten der gemessenen Beschleunigung
subplot(2,2,1);
plot(time,beschl_kraft(:,1));
ylabel('Voltage');
xlabel('Time');
title('gemessene Beschleunigung');
grid on;
grid minor;

subplot(2,2,2);                                        %Plotten der gemessenen Kraft
plot(time,beschl_kraft(:,2));
ylabel('Voltage');
xlabel('Time');
title('gemessene Kraft');
grid on;
grid minor;

subplot(2,2,3);                                        %Plotten der errechneten Beschleunigung
plot(time,B);
ylabel('Beschleunigung in m/s');
xlabel('Time');
title('errechnete Beschleunigung');
grid on;
grid minor;

subplot(2,2,4);                                        %Plotten der gemessenen Kraft
plot(time,F);
ylabel('Kraft in N');
xlabel('Time');
title('errechnete Kraft');
grid on;
grid minor;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Berechnung
blocksize = s.Rate;                                                             %Blocksize des Spektrogramm ist die Abtastrate (wegen Frequenzauflsung)
noverlap = ceil(blocksize*2/3);                                                 %berlappung der Blcke ist 2/3 der Blocksize
window = hann(blocksize)*blocksize/sum(hann(blocksize));                        %Fensterfunktion fr Spektrogramm ist Hanning

[fft_aus,f,t,p] = spectrogram(output_data,window,noverlap,blocksize,s.Rate);    %Spektrogramm des Ausgangssignals

[fft_ist1,f,t,p] = spectrogram(B,window,noverlap,blocksize,s.Rate);             %Spektrogramm des gemessenen Beschleunigungssignals
[fft_ist2,f,t,p] = spectrogram(F,window,noverlap,blocksize,s.Rate);             %Spektrogramm des gemessenen Kraftsiganls



h=fft_ist1./fft_aus;                                                            %bertragungsfunktion aus FFT der Beschl. / FFT des Ausgangssignals
h1=fft_ist2./fft_aus;                                                           %bertragungsfunktion aus FFT der Kraft / FFT des Ausgangssignals


figure(4);                                                                      %Plotten der bertragungsfunktion von Beschleunigung ber Frequenz
subplot(3,1,1);
plot(f,(mean(abs(h),2)),'LineWidth',2);
ylabel('Beschleunigungswerte der F in [m/s/V]');
xlabel('Frequenz in Hz');
title('bertragungsfunktion der Beschleunigung');
grid on;
grid minor;

subplot(3,1,2);                                                                 %Plotten der bertragungsfunktion von Kraft ber Frequenz
plot(f,(mean(abs(h1),2)),'LineWidth',2);
ylabel('Kraftwerte der F in [N/V]');
xlabel('Frequenz in Hz');
title('bertragungsfunktion der Kraft');
grid on;
grid minor;


UF_Beschl=(mean(abs(h),2));                                                     %Normieren der bertragungsfunktion der Beschleunigung
UF_Kraft=(mean(abs(h1),2));                                                     %Normieren der bertragungsfunktion der Kraft

UF_Weg1=UF_Beschl./(2*pi*f).^2;                                                 %Berechnung der Werte der bertragungsfunktion des Weges
UF_Weg=UF_Weg1*1000000;                                                         %Umrechnung auf Mikrometer


subplot(3,1,3);                                                                 %Plotten der bertragungsfunktion von Weg ber Frequenz
plot(f,UF_Weg,'LineWidth',2);
ylabel('Wegwerte der F in [Mikrometer/V]');
xlabel('Frequenz in Hz');
title('bertragungsfunktion des Weges');
grid on;
grid minor;


V=input('Bitte die Regelgre eingeben (Beschleunigung =1 , Weg =2 , Kraft =3):');  %Eingabe der Regelgre
Startf=input('Bitte die Startfrequenz eingeben [Hz]:');                             %Eingabe der Startfrequenz
SollGroesse=input('Bitte die Sollgre eingeben ([m/s] [N] oder Mikrometer):');    %Eingabe der Sollgre

if (98<=Startf>=102)                                                                %Die bertragungsfunktion weist bei ca. 100Hz eine Resonanz auf,
    Startf=95;                                                                      %welche hierdurch in der Berechnung vermieden werden kann.
end

switch V
    case 1
        UF=UF_Beschl(Startf+1);                                                 %Berechnung des bertragungswertes der Beschleunigung bei Startfrequenz
    case 2
        UF=UF_Weg(Startf+1);                                                    %Berechnung des bertragungswertes des Weges bei Startfrequenz
    case 3
        UF=UF_Kraft(Startf+1);                                                  %Berechnung des bertragungswertes der Kraft bei Startfrequenz
end

%Berechnung Anfangsamplitude a

a=SollGroesse/UF;                                                               %optimale Anfangsamplitude ist Sollgre/bertragungswert

if (a<0.5)                                                                      
    disp('Regelung nicht mglich, da Startamplitude auerhalb gltigen Bereich!');                  %Wenn a kleiner als 0,5 ist, ist die Regelung nicht mglich
    disp('Bitte Leistungsverstrker nachstellen (Verringern) oder Eingabeparameter anpassen!');
    
elseif (a>9.5)
    disp('Regelung nicht mglich, da Startamplitude auerhalb gltigen Bereich!');                  %Wenn a grer als 9,5 ist, ist die Regelung nicht mglich
    disp('Bitte Leistungsverstrker nachstellen (Erhhen) oder Eingabeparameter anpassen!');

else

disp('Ihre optimale Spartamplitude a in [V] betrgt:');                                             %Wenn Regelung mglich, dann Rckgabe der optimalen Startamplitude in V
disp(a);
end
